#include <Arduino.h>
//knihovna na Wi-Fi
#include <WiFi.h> 
//knihovna na MQTT
#include <PubSubClient.h>

//WIFI konstanta pro jméno síťě a heslo
const char* ssid = "Cukrovar1";
const char* password = "1158jarda";

//MQTT
//konstanta pro ip hostujeme MQTT(v našem případě na raspberry pi)
#define mqtt_server "192.168.1.113"
WiFiClient wifiClientEsp;
PubSubClient client(wifiClientEsp);

//MQTT Topics
//definování našeho topicu 
#define mqttLedOnOff "esp32Three/Led/value"
#define mqttBuzzer "esp32Three/Buzzer/value"

//definování pinu na kterém je led dioda
#define PIN_LED  32
//definování pinu na kterém je aktivní pzučák
#define PIN_BUZZER 33

// metoda wi-fi nastavení
void wifiSetup(){
  Serial.println();
  Serial.print("Connecting to ");
  //Vypíše nazev sítě ke které se napojuje
  Serial.println(ssid);

  //Přepne wi-fi mode do station nastavení
  WiFi.mode(WIFI_STA);
  WiFi.disconnect();
  delay(2000);

  //pokusí se připojit pomocí nazvu sítě a hesla
  WiFi.begin(ssid, password);

  //Opakuj dokud eps není připojeno k wi-fi
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
 
  Serial.println("");
  Serial.println("Wi-Fi connected");  
  Serial.println("IP address: ");
  //Vypíše IP adresu na které se připojil do lokální sítě
  Serial.println(WiFi.localIP());
}

//tato metoda příjmá tři paramentry a to je topic, zprávu a délku zprávy
void mqttCallback(char* topic, byte* message, unsigned int length) {
  Serial.print("MQTT message received on topic: ");
  //vypsaní topicu o jaký se jedná
  Serial.print(topic);
  Serial.print(". Message: ");
  String messageTemp;
  //vypsaní zprávy po characteru a uložení do proměné
  for (int i = 0; i < length; i++) {
    messageTemp += (char)message[i];
  }
  //vypsání přijatého příkazu
  Serial.println(messageTemp);
  //rozhodnutí i jaký příkaz se jedná a jeho vykonání
  if (messageTemp == "ledOn"){
     digitalWrite(PIN_LED, HIGH);
  }
  if (messageTemp == "ledOff"){
     digitalWrite(PIN_LED, LOW);
  }
  if (messageTemp == "buzz"){
     for (char i = 0; i<100; i++){
          digitalWrite(PIN_BUZZER,HIGH);
          delay(1);
          digitalWrite(PIN_BUZZER,LOW);
          delay(1);
     }
  }
}

void setup() {  
  //rychlost na 115200 baudů
  Serial.begin(115200);
  // definování bastavení pinů na výstup
  pinMode(PIN_LED, OUTPUT);
  pinMode(PIN_BUZZER, OUTPUT);
  //volání metody na nastavení wi-fi
  wifiSetup();

  //Nastavení serveru MQTT pro klienta, ip adressa a port na kterém beží
  client.setServer(mqtt_server, 1883);
  //nastavení metody která obluhuje mqqt zpětného volání
  client.setCallback(mqttCallback);
  
}
//metoda pro znovu připojení k MQTT a připojení
void reconnectAndConnectMQTT() {
  int counter = 0;
  // opakuj a pokud není klient připojen k MQTT
  while (!client.connected()) {
    //pokud se 5 křát nepřipojí
    if (counter==5){
      //restartujeme ESP
      ESP.restart();
    }
    counter+=1;
    Serial.print("Attempting MQTT connection... ");
   
    if (client.connect("esp32Three")) {
      Serial.println("connected");
      //topics o kterých chceme ze serveru dostavat data
      client.subscribe(mqttLedOnOff);
      client.subscribe(mqttBuzzer);
    } else {
      Serial.print("failed, rc=");
      //výpis chybi proč se nemůže připojit
      Serial.print(client.state());
      Serial.println(" try again in 5 seconds");
      delay(5000);
    }
  }
  client.subscribe(mqttLedOnOff);
  client.subscribe(mqttBuzzer);
}
//metoda pomocí které můžeme testovat zda komponenty fungují
void controlBySerial (){
   if(Serial.available()){
        String command = Serial.readStringUntil('\n');
        
        Serial.printf("Command received %s \n", command);

        if(command.equals("ledOn")){
            digitalWrite(PIN_LED, HIGH);
        } else if(command.equals("ledOff")){
            digitalWrite(PIN_LED, LOW);
        } else if(command.equals("buzz")){
            for (char i = 0; i<100; i++){
              digitalWrite(PIN_BUZZER,HIGH);
              delay(1);
              digitalWrite(PIN_BUZZER,LOW);
              delay(1);
            }
        } else{
            Serial.println("Invalid command");
        }
    }
}
void loop() {
  if (!client.connected()){
    reconnectAndConnectMQTT();
  }
  
  controlBySerial();
  
  client.loop();
}
