#include <Arduino.h>
//knihovna na Wi-Fi
#include <WiFi.h> 
//knihovna na MQTT
#include <PubSubClient.h>

//WIFI konstanta pro jméno síťě a heslo
const char* ssid = "Cukrovar1";
const char* password = "1158jarda";

//MQTT
//konstanta pro ip hostujeme MQTT(v našem případě na raspberry pi)
#define mqtt_server "192.168.1.113"
WiFiClient wifiClientEsp;
PubSubClient client(wifiClientEsp);

//MQTT Topics
//definování našeho topicu 
#define mqttLight "esp32Two/photoresitor/value"

//definování pinu na kterém přímáme data
#define PIN_PHOTORESISTOR 39

//globální proměná pro data ze senzoru
int analogValue;

//proměná pro místo funkce delay
unsigned long millisNow = 0; 
unsigned int sendDelay = 1000; 

// metoda wi-fi nastavení
void wifiSetup(){
  Serial.println();
  Serial.print("Connecting to ");
  //Vypíše nazev sítě ke které se napojuje
  Serial.println(ssid);

  //Přepne wi-fi mode do station nastavení
  WiFi.mode(WIFI_STA);
  WiFi.disconnect();
  delay(2000);

  //pokusí se připojit pomocí nazvu sítě a hesla
  WiFi.begin(ssid, password);

  //Opakuj dokud eps není připojeno k wi-fi
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
 
  Serial.println("");
  Serial.println("Wi-Fi connected");  
  Serial.println("IP address: ");
  //Vypíše IP adresu na které se připojil do lokální sítě
  Serial.println(WiFi.localIP());
}

void setup() {
  //rychlost na 115200 baudů
  Serial.begin(115200);
  // definování pin nastavení na vstup
  pinMode(PIN_PHOTORESISTOR, INPUT);
  //volání metody na nastavení wi-fi
  wifiSetup();
  //Nastavení serveru MQTT pro klienta, ip adressa a port na kterém beží
  client.setServer(mqtt_server, 1883);
}

//metoda pro znovu připojení k MQTT a připojení
void reconnectAndConnectMQTT() {
  int counter = 0;
  // opakuj a pokud není klient připojen k MQTT
  while (!client.connected()) {
    //pokud se 5 křát nepřipojí
    if (counter==5){
      //restartujeme ESP
      ESP.restart();
    }
    counter+=1;
    Serial.print("Attempting MQTT connection... ");
   
    if (client.connect("esp32Onne")) {
      Serial.println("connected");
    } else {
      Serial.print("failed, rc=");
      //výpis chybi proč se nemůže připojit
      Serial.print(client.state());
      Serial.println(" try again in 5 seconds");
      delay(5000);
    }
  }
}

//zde máme metodu pro snímání dat ze senzoru a vypis do seriového monitoru
void getValuesFromSensor() {
  analogValue = analogRead(PIN_PHOTORESISTOR);

  Serial.print("Analog reading = ");
  Serial.print(analogValue);
  
  if (analogValue < 10) {
    Serial.println(" - Dark");
  } else if (analogValue < 500) {
    Serial.println(" - Light");
  } else {
    Serial.println(" - Very bright");
  }
}

void loop() {
  // po každé kontrolujeme jesli je client připojen na MQTT server
  if (!client.connected()){
    //pokud není tak se znovu chceme připojit
    reconnectAndConnectMQTT();
  }
  //tato podmínka slouží pro publikování se zpožděním
  if (millis() > millisNow + sendDelay) {
    millisNow = millis();
    getValuesFromSensor();  
    client.publish(mqttLight, String(analogValue).c_str(),true);
  }
  
  client.loop();
}
